@@ -14,8 +14,9 @@ if __name__ == '__main__':
   ToolchainProfiler.record_process_start()
 
 import difflib
-import os, sys, json, optparse, subprocess, re, time, logging
+import os, sys, json, argparse, subprocess, re, time, logging
 import shutil
+from collections import OrderedDict
 
 from tools import shared
 from tools import jsrun, cache as cache_module, tempfiles
@@ -46,7 +47,7 @@ if STDERR_FILE:
 def quoter(settings):
   def quote(prop):
     if settings['USE_CLOSURE_COMPILER'] == 2:
-      return ''.join(map(lambda p: "'" + p + "'", prop.split('.')))
+      return ''.join(["'" + p + "'" for p in prop.split('.')])
     else:
       return prop
   return quote
@@ -54,7 +55,7 @@ def quoter(settings):
 def access_quoter(settings):
   def access_quote(prop):
     if settings['USE_CLOSURE_COMPILER'] == 2:
-      return ''.join(map(lambda p: "['" + p + "']", prop.split('.')))
+      return ''.join(["['" + p + "']" for p in prop.split('.')])
     else:
       return '.' + prop
   return access_quote
@@ -144,8 +145,8 @@ def parse_backend_output(backend_output, DEBUG):
 
   try:
     #if DEBUG: print >> sys.stderr, "METAraw", metadata_raw
-    metadata = json.loads(metadata_raw)
-  except Exception, e:
+    metadata = json.loads(metadata_raw, object_pairs_hook=OrderedDict)
+  except Exception as e:
     logging.error('emscript: failure to parse metadata output from compiler backend. raw output is: \n' + metadata_raw)
     raise e
 
@@ -159,9 +160,9 @@ def fixup_metadata_tables(metadata, settings):
   # if emulating pointer casts, force all tables to the size of the largest
   if settings['EMULATE_FUNCTION_POINTER_CASTS']:
     max_size = 0
-    for k, v in metadata['tables'].iteritems():
+    for k, v in metadata['tables'].items():
       max_size = max(max_size, v.count(',')+1)
-    for k, v in metadata['tables'].iteritems():
+    for k, v in metadata['tables'].items():
       curr = v.count(',')+1
       if curr < max_size:
         metadata['tables'][k] = v.replace(']', (',0'*(max_size - curr)) + ']')
@@ -174,7 +175,7 @@ def fixup_metadata_tables(metadata, settings):
 def fixup_functions(funcs, metadata, settings):
   # function table masks
   table_sizes = {}
-  for k, v in metadata['tables'].iteritems():
+  for k, v in metadata['tables'].items():
     table_sizes[k] = str(v.count(',')) # undercounts by one, but that is what we want
     #if settings['ASSERTIONS'] >= 2 and table_sizes[k] == 0:
     #  print >> sys.stderr, 'warning: no function pointers with signature ' + k + ', but there is a call, which will abort if it occurs (this can result from undefined behavior, check for compiler warnings on your source files and consider -Werror)'
@@ -210,7 +211,7 @@ def compiler_glue(metadata, settings, libraries, compiler_engine, temp_files, DE
       break
 
   # FIXME: do these one by one as normal js lib funcs
-  metadata['declares'] = filter(lambda i64_func: i64_func not in ['getHigh32', 'setHigh32'], metadata['declares'])
+  metadata['declares'] = [i64_func for i64_func in metadata['declares'] if i64_func not in ['getHigh32', 'setHigh32']]
 
   optimize_syscalls(metadata['declares'], settings, DEBUG)
   update_settings_glue(settings, metadata)
@@ -272,7 +273,7 @@ def function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data,
   else:
     pre_tables = ''
 
-  function_table_sigs = function_table_data.keys()
+  function_table_sigs = list(function_table_data.keys())
 
   in_table, debug_tables, function_tables_defs = make_function_tables_defs(
     implemented_functions, all_implemented, function_table_data, settings, metadata)
@@ -299,7 +300,7 @@ def function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data,
     global_vars = metadata['externs']
   else:
     global_vars = [] # linkable code accesses globals through function calls
-  global_funcs = list(set([key for key, value in forwarded_json['Functions']['libraryFunctions'].iteritems() if value != 2])
+  global_funcs = sorted(set([key for key, value in forwarded_json['Functions']['libraryFunctions'].items() if value != 2])
                       .difference(set(global_vars)).difference(implemented_functions))
   if settings['RELOCATABLE']:
     global_funcs += ['g$' + extern for extern in metadata['externs']]
@@ -329,7 +330,7 @@ def function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data,
 
   if DEBUG:
     logging.debug('asm text sizes' + str([
-      map(len, funcs_js), len(asm_setup), len(asm_global_vars), len(asm_global_funcs), len(pre_tables),
+      list(map(len, funcs_js)), len(asm_setup), len(asm_global_vars), len(asm_global_funcs), len(pre_tables),
       len('\n'.join(function_tables_impls)), len(function_tables_defs) + (function_tables_defs.count('\n') * len('  ')),
       len(exports), len(the_global), len(sending), len(receiving)]))
     logging.debug('  emscript: python processing: function tables and exports took %s seconds' % (time.time() - t))
@@ -340,7 +341,7 @@ def function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data,
 
 
 def finalize_output(outfile, post, function_table_data, bundled_args, metadata, settings, DEBUG):
-  function_table_sigs = function_table_data.keys()
+  function_table_sigs = list(function_table_data.keys())
   module = create_module(function_table_sigs, metadata, settings, *bundled_args)
 
   if DEBUG:
@@ -393,7 +394,7 @@ def create_module(function_table_sigs, metadata, settings,
   if settings['SIDE_MODULE']:
     module.append('''
 Runtime.registerFunctions(%(sigs)s, Module);
-''' % { 'sigs': str(map(str, function_table_sigs)) })
+''' % { 'sigs': str(list(map(str, function_table_sigs))) })
 
   return module
 
@@ -505,11 +506,11 @@ def update_settings_glue(settings, metadata):
     logging.warning('disabling asm.js validation due to use of non-supported features: ' + metadata['cantValidate'])
     settings['ASM_JS'] = 2
 
-  settings['DEFAULT_LIBRARY_FUNCS_TO_INCLUDE'] = list(
-    set(settings['DEFAULT_LIBRARY_FUNCS_TO_INCLUDE'] + map(shared.JS.to_nice_ident, metadata['declares'])).difference(
-      map(lambda x: x[1:], metadata['implementedFunctions'])
+  settings['DEFAULT_LIBRARY_FUNCS_TO_INCLUDE'] = sorted(
+    set(settings['DEFAULT_LIBRARY_FUNCS_TO_INCLUDE'] + list(map(shared.JS.to_nice_ident, metadata['declares']))).difference(
+      [x[1:] for x in metadata['implementedFunctions']]
     )
-  ) + map(lambda x: x[1:], metadata['externs'])
+  ) + [x[1:] for x in metadata['externs']]
 
   if metadata['simd']:
     settings['SIMD'] = 1
@@ -539,7 +540,7 @@ def compile_settings(compiler_engine, settings, libraries, temp_files):
 
 
 def memory_and_global_initializers(pre, metadata, mem_init, settings):
-  global_initializers = str(', '.join(map(lambda i: '{ func: function() { %s() } }' % i, metadata['initializers'])))
+  global_initializers = str(', '.join(['{ func: function() { %s() } }' % i for i in metadata['initializers']]))
 
   if settings['SIMD'] == 1:
     pre = open(path_from_root(os.path.join('src', 'ecmascript_simd.js'))).read() + '\n\n' + pre
@@ -593,7 +594,7 @@ def get_all_exported_functions(function_table_data, settings):
 
 
 def get_all_implemented(forwarded_json, metadata):
-  return metadata['implementedFunctions'] + forwarded_json['Functions']['implementedFunctions'].keys() # XXX perf?
+  return metadata['implementedFunctions'] + list(forwarded_json['Functions']['implementedFunctions'].keys()) # XXX perf?
 
 
 def check_all_implemented(all_implemented, pre, settings):
@@ -635,33 +636,51 @@ def get_exported_implemented_functions(all_exported_functions, all_implemented,
       funcs += ['setTempRet0', 'getTempRet0']
     if not (settings['BINARYEN'] and settings['SIDE_MODULE']):
       funcs += ['setThrew']
-  return list(set(funcs))
+  return sorted(set(funcs))
 
 
 def get_implemented_functions(metadata):
   return set(metadata['implementedFunctions'])
 
+def proxy_debug_print(call_type, settings):
+  if shared.Settings.PTHREADS_DEBUG:
+    if call_type == 'sync_on_main_thread_': return 'Runtime.warnOnce("sync proxying function " + code);';
+    elif call_type == 'async_on_main_thread_': return 'Runtime.warnOnce("async proxying function " + code);';
+  return ''
 
 def include_asm_consts(pre, forwarded_json, metadata, settings):
   if settings['BINARYEN'] and settings['SIDE_MODULE']:
     assert len(metadata['asmConsts']) == 0, 'EM_ASM is not yet supported in shared wasm module (it cannot be stored in the wasm itself, need some solution)'
 
-  asm_consts, all_sigs = all_asm_consts(metadata)
+  asm_consts, all_sigs, call_types = all_asm_consts(metadata)
   asm_const_funcs = []
-  for sig in set(all_sigs):
-    forwarded_json['Functions']['libraryFunctions']['_emscripten_asm_const_' + sig] = 1
+  for s in range(len(all_sigs)):
+    sig = all_sigs[s]
+    call_type = call_types[s] if s < len(call_types) else ''
+    if '_emscripten_asm_const_' + call_type + sig in forwarded_json['Functions']['libraryFunctions']: continue # Only one invoker needs to be emitted for each ASM_CONST (signature x call_type) item
+    forwarded_json['Functions']['libraryFunctions']['_emscripten_asm_const_' + call_type + sig] = 1
     args = ['a%d' % i for i in range(len(sig)-1)]
     all_args = ['code'] + args
+    proxy_function = ''
+    if shared.Settings.USE_PTHREADS:
+      if call_type == 'sync_on_main_thread_': proxy_function = '_emscripten_sync_run_in_browser_thread_' + sig
+      elif call_type == 'async_on_main_thread_': proxy_function = '_emscripten_async_run_in_browser_thread_' + sig
+
+    # In proxied function calls, positive integers 1, 2, 3, ... denote pointers to regular C compiled functions. Negative integers -1, -2, -3, ... denote indices to EM_ASM() blocks, so remap the EM_ASM() indices from 0, 1, 2, ... over to the negative integers starting at -1.
+    proxy_args = '-1 - ' + ','.join(all_args)
+
+    if proxy_function: proxy_to_main_thread = '  if (ENVIRONMENT_IS_PTHREAD) { ' + proxy_debug_print(call_type, settings) + 'return ' + proxy_function + '(' + proxy_args + '); } \n'
+    else: proxy_to_main_thread = ''
     asm_const_funcs.append(r'''
 function _emscripten_asm_const_%s(%s) {
-  return ASM_CONSTS[code](%s);
-}''' % (sig.encode('utf-8'), ', '.join(all_args), ', '.join(args)))
+%s  return ASM_CONSTS[code](%s);
+}''' % (call_type + sig.encode('utf-8'), ', '.join(all_args), proxy_to_main_thread, ', '.join(args)))
 
   asm_consts_text = '\nvar ASM_CONSTS = [' + ',\n '.join(asm_consts) + '];\n'
   asm_funcs_text = '\n'.join(asm_const_funcs) + '\n'
 
   body_marker = '// === Body ==='
-  return pre.replace(body_marker, body_marker + '\n' + asm_consts_text + asm_funcs_text)
+  return pre.replace(body_marker, body_marker + '\n' + asm_consts_text + asm_funcs_text.encode('utf-8'))
 
 # Test if the parentheses at body[openIdx] and body[closeIdx] are a match to each other.
 def parentheses_match(body, openIdx, closeIdx):
@@ -689,9 +708,11 @@ def trim_asm_const_body(body):
 def all_asm_consts(metadata):
   asm_consts = [0]*len(metadata['asmConsts'])
   all_sigs = []
-  for k, v in metadata['asmConsts'].iteritems():
+  all_call_types = []
+  for k, v in metadata['asmConsts'].items():
     const = v[0].encode('utf-8')
     sigs = v[1]
+    call_types = v[2] if len(v) >= 3 else None
     const = trim_asm_const_body(const)
     const = '{ ' + const + ' }'
     args = []
@@ -701,7 +722,8 @@ def all_asm_consts(metadata):
     const = 'function(' + ', '.join(args) + ') ' + const
     asm_consts[int(k)] = const
     all_sigs += sigs
-  return asm_consts, all_sigs
+    if call_types: all_call_types += call_types
+  return asm_consts, all_sigs, all_call_types
 
 
 def unfloat(s):
@@ -725,11 +747,11 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
   # when emulating function pointer casts, we need to know what is the target of each pointer
   if settings['EMULATE_FUNCTION_POINTER_CASTS']:
     function_pointer_targets = {}
-    for sig, table in function_table_data.iteritems():
+    for sig, table in function_table_data.items():
       start = table.index('[')
       end = table.rindex(']')
       body = table[start+1:end].split(',')
-      parsed = map(lambda x: x.strip(), body)
+      parsed = [x.strip() for x in body]
       for i in range(len(parsed)):
         if parsed[i] != '0':
           assert i not in function_pointer_targets
@@ -769,7 +791,7 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
           return item
         in_table.add(item)
         return "asm['" + item + "']"
-      body = map(receive, body)
+      body = list(map(receive, body))
     for j in range(settings['RESERVED_FUNCTION_POINTERS']):
       curr = 'jsCall_%s_%s' % (sig, j)
       body[settings['FUNCTION_POINTER_ALIGNMENT'] * (1 + j)] = curr
@@ -788,7 +810,7 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
           def make_emulated_param(i):
             if i >= len(sig): return shared.JS.make_initializer(proper_sig[i], settings) # extra param, just send a zero
             return shared.JS.make_coercion('p%d' % (i-1), proper_sig[i], settings, convert_from=sig[i])
-          proper_code = proper_target + '(' + ','.join(map(lambda i: make_emulated_param(i+1), range(len(proper_sig)-1))) + ')'
+          proper_code = proper_target + '(' + ','.join([make_emulated_param(i+1) for i in range(len(proper_sig)-1)]) + ')'
           if proper_sig[0] != 'v':
             # proper sig has a return, which the wrapper may or may not use
             proper_code = shared.JS.make_coercion(proper_code, proper_sig[0], settings)
@@ -835,7 +857,7 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
     body = ','.join(map(fix_item, body))
     return ('\n'.join(Counter.pre), ''.join([raw[:start+1], body, raw[end:]]))
 
-  infos = [make_table(sig, raw) for sig, raw in function_table_data.iteritems()]
+  infos = [make_table(sig, raw) for sig, raw in function_table_data.items()]
   Counter.pre = []
 
   function_tables_defs = '\n'.join([info[0] for info in infos]) + '\n'
@@ -854,7 +876,7 @@ def math_fix(g):
 
 def make_function_tables_impls(function_table_data, settings):
   function_tables_impls = []
-  for sig, table in function_table_data.iteritems():
+  for sig, table in function_table_data.items():
     args = ','.join(['a' + str(i) for i in range(1, len(sig))])
     arg_coercions = ' '.join(['a' + str(i) + '=' + shared.JS.make_coercion('a' + str(i), sig[i], settings) + ';' for i in range(1, len(sig))])
     coerced_args = ','.join([shared.JS.make_coercion('a' + str(i), sig[i], settings) for i in range(1, len(sig))])
@@ -890,7 +912,7 @@ def create_mftCall_funcs(function_table_data, settings):
   mftCall_funcs = []
   if settings.get('EMULATED_FUNCTION_POINTERS'):
     if settings.get('RELOCATABLE') and not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
-      for sig, table in function_table_data.iteritems():
+      for sig, table in function_table_data.items():
         params = ','.join(['ptr'] + ['p%d' % p for p in range(len(sig)-1)])
         coerced_params = ','.join([shared.JS.make_coercion('ptr', 'i', settings)] + [shared.JS.make_coercion('p%d', unfloat(sig[p+1]), settings) % p for p in range(len(sig)-1)])
         coercions = ';'.join(['ptr = ptr | 0'] + ['p%d = %s' % (p, shared.JS.make_coercion('p%d' % p, unfloat(sig[p+1]), settings)) for p in range(len(sig)-1)]) + ';'
@@ -993,7 +1015,7 @@ def global_simd_funcs(access_quote, metadata, settings):
 
   def generate_symbols(types, funcs):
     symbols = ['  var SIMD_' + ty + '_' + g + '=SIMD_' + ty + access_quote(g) + ';\n' for ty in types for g in funcs]
-    symbols = filter(lambda x: not string_contains_any(x, nonexisting_simd_symbols), symbols)
+    symbols = [x for x in symbols if not string_contains_any(x, nonexisting_simd_symbols)]
     return ''.join(symbols)
 
   simd_func_text += generate_symbols(simd['int_types'], simd['int_funcs'])
@@ -1101,7 +1123,7 @@ def provide_fround(settings):
 
 
 def create_asm_setup(debug_tables, function_table_data, metadata, settings):
-  function_table_sigs = function_table_data.keys()
+  function_table_sigs = list(function_table_data.keys())
 
   asm_setup = ''
   if settings['ASSERTIONS'] >= 2:
@@ -1117,7 +1139,7 @@ def create_asm_setup(debug_tables, function_table_data, metadata, settings):
         return 0
       return table_contents.count(',') + 1
 
-    table_total_size = sum(map(table_size, function_table_data.values()))
+    table_total_size = sum(map(table_size, list(function_table_data.values())))
     asm_setup += "\nModule['wasmTableSize'] = %d;\n" % table_total_size
     if not settings['EMULATED_FUNCTION_POINTERS']:
       asm_setup += "\nModule['wasmMaxTableSize'] = %d;\n" % table_total_size
@@ -1228,16 +1250,16 @@ def create_exports(exported_implemented_functions, in_table, function_table_data
     asm_runtime_funcs.append('setAsync')
   all_exported = exported_implemented_functions + asm_runtime_funcs + function_tables(function_table_data, settings)
   if settings['EMULATED_FUNCTION_POINTERS']:
-    all_exported = list(set(all_exported).union(in_table))
+    all_exported += in_table
   exports = []
-  for export in set(all_exported):
+  for export in sorted(set(all_exported)):
     exports.append(quote(export) + ": " + export)
   if settings['BINARYEN'] and settings['SIDE_MODULE']:
     # named globals in side wasm modules are exported globals from asm/wasm
-    for k, v in metadata['namedGlobals'].iteritems():
+    for k, v in metadata['namedGlobals'].items():
       exports.append(quote('_' + str(k)) + ': ' + str(v))
     # aliases become additional exports
-    for k, v in metadata['aliases'].iteritems():
+    for k, v in metadata['aliases'].items():
       exports.append(quote(str(k)) + ': ' + str(v))
   return '{ ' + ', '.join(exports) + ' }'
 
@@ -1324,7 +1346,7 @@ for (var named in NAMED_GLOBALS) {
   Module['_' + named] = gb + NAMED_GLOBALS[named];
 }
 Module['NAMED_GLOBALS'] = NAMED_GLOBALS;
-''' % ', '.join('"' + k + '": ' + str(v) for k, v in metadata['namedGlobals'].iteritems())
+''' % ', '.join('"' + k + '": ' + str(v) for k, v in metadata['namedGlobals'].items())
     if settings['BINARYEN']:
       # wasm side modules are pure wasm, and cannot create their g$..() methods, so we help them out
       # TODO: this works if we are the main module, but if the supplying module is later, it won't, so
@@ -1337,7 +1359,7 @@ for (var named in NAMED_GLOBALS) {
   })(named);
 }
 '''
-    named_globals += ''.join(["Module['%s'] = Module['%s']\n" % (k, v) for k, v in metadata['aliases'].iteritems()])
+    named_globals += ''.join(["Module['%s'] = Module['%s']\n" % (k, v) for k, v in metadata['aliases'].items()])
   return named_globals
 
 
@@ -1495,10 +1517,10 @@ def create_asm_start_pre(asm_setup, the_global, sending, metadata, settings):
   module_library = module_get.format(access=access_quote('asmLibraryArg'), val=sending)
 
   asm_function_top = ('// EMSCRIPTEN_START_ASM\n'
-                      'var asm = (function(global, env, buffer) {')
+                      'var asm = (/** @suppress {uselessCode} */ function(global, env, buffer) {')
 
   use_asm = "'almost asm';"
-  if not metadata.get('hasInlineJS') and settings['ASM_JS'] == 1:
+  if settings['ASM_JS'] == 1:
     use_asm = "'use asm';"
 
   lines = [
@@ -1732,7 +1754,7 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
 
   # memory and global initializers
 
-  global_initializers = str(', '.join(map(lambda i: '{ func: function() { %s() } }' % i, metadata['initializers'])))
+  global_initializers = str(', '.join(['{ func: function() { %s() } }' % i for i in metadata['initializers']]))
 
   staticbump = metadata['staticBump']
   while staticbump % 16 != 0: staticbump += 1
@@ -1831,7 +1853,7 @@ def create_exported_implemented_functions_wasm(pre, forwarded_json, metadata, se
   all_exported_functions = set(shared.expand_response(settings['EXPORTED_FUNCTIONS'])) # both asm.js and otherwise
   for additional_export in settings['DEFAULT_LIBRARY_FUNCS_TO_INCLUDE']: # additional functions to export from asm, if they are implemented
     all_exported_functions.add('_' + additional_export)
-  all_implemented = metadata['implementedFunctions'] + forwarded_json['Functions']['implementedFunctions'].keys() # XXX perf?
+  all_implemented = metadata['implementedFunctions'] + list(forwarded_json['Functions']['implementedFunctions'].keys()) # XXX perf?
 
   export_bindings = settings['EXPORT_BINDINGS']
   export_all = settings['EXPORT_ALL']
@@ -1855,7 +1877,7 @@ def create_exported_implemented_functions_wasm(pre, forwarded_json, metadata, se
 def create_asm_consts_wasm(forwarded_json, metadata):
   asm_consts = [0]*len(metadata['asmConsts'])
   all_sigs = []
-  for k, v in metadata['asmConsts'].iteritems():
+  for k, v in metadata['asmConsts'].items():
     const = v[0].encode('utf-8')
     sigs = v[1]
     const = trim_asm_const_body(const)
@@ -1907,7 +1929,7 @@ def create_sending_wasm(invoke_funcs, forwarded_json, metadata, settings):
     global_vars = [] # linkable code accesses globals through function calls
 
   implemented_functions = set(metadata['implementedFunctions'])
-  global_funcs = list(set([key for key, value in forwarded_json['Functions']['libraryFunctions'].iteritems() if value != 2]).difference(set(global_vars)).difference(implemented_functions))
+  global_funcs = list(set([key for key, value in forwarded_json['Functions']['libraryFunctions'].items() if value != 2]).difference(set(global_vars)).difference(implemented_functions))
 
   send_items = basic_funcs + invoke_funcs + global_funcs + basic_vars + global_vars
   def math_fix(g):
@@ -2024,7 +2046,7 @@ def create_s2wasm_args(temp_s):
 def load_metadata(metadata_raw):
   try:
     metadata_json = json.loads(metadata_raw)
-  except Exception, e:
+  except Exception as e:
     logging.error('emscript: failure to parse metadata output from s2wasm. raw output is: \n' + metadata_raw)
     raise e
 
@@ -2038,11 +2060,11 @@ def load_metadata(metadata_raw):
     'exports': [],
   }
 
-  for k, v in metadata_json.iteritems():
+  for k, v in metadata_json.items():
     metadata[k] = v
 
   # Initializers call the global var version of the export, so they get the mangled name.
-  metadata['initializers'] = map(asmjs_mangle, metadata['initializers'])
+  metadata['initializers'] = list(map(asmjs_mangle, metadata['initializers']))
 
   return metadata
 
@@ -2085,7 +2107,7 @@ def add_metadata_from_wast(metadata, wast):
         assert False, 'Unhandled export type "%s"' % export_type
 
   # we emit those ourselves
-  metadata['declares'] = filter(lambda x: not x.startswith('emscripten_asm_const'), metadata['declares'])
+  metadata['declares'] = [x for x in metadata['declares'] if not x.startswith('emscripten_asm_const')]
 
 
 def create_invoke_wrappers(invoke_funcs):
@@ -2167,50 +2189,54 @@ def _main(args=None):
         args[index:index+1] = response_file_args
         break
 
-  parser = optparse.OptionParser(
-    usage='usage: %prog [-h] [-H HEADERS] [-o OUTFILE] [-c COMPILER_ENGINE] [-s FOO=BAR]* infile',
+  parser = argparse.ArgumentParser(
+    usage='%(prog)s [-h] [-H HEADERS] [-o OUTFILE] [-c COMPILER_ENGINE] [-s FOO=BAR]* infile',
     description=('You should normally never use this! Use emcc instead. '
                  'This is a wrapper around the JS compiler, converting .ll to .js.'),
     epilog='')
-  parser.add_option('-H', '--headers',
+  parser.add_argument('-H', '--headers',
                     default=[],
                     action='append',
                     help='System headers (comma separated) whose #defines should be exposed to the compiled code.')
-  parser.add_option('-L', '--libraries',
+  parser.add_argument('-L', '--libraries',
                     default=[],
                     action='append',
                     help='Library files (comma separated) to use in addition to those in emscripten src/library_*.')
-  parser.add_option('-o', '--outfile',
+  parser.add_argument('-o', '--outfile',
                     default=sys.stdout,
                     help='Where to write the output; defaults to stdout.')
-  parser.add_option('-c', '--compiler',
+  parser.add_argument('-c', '--compiler',
                     default=None,
                     help='Which JS engine to use to run the compiler; defaults to the one in ~/.emscripten.')
-  parser.add_option('-s', '--setting',
+  parser.add_argument('-s', '--setting',
                     dest='settings',
                     default=[],
                     action='append',
                     metavar='FOO=BAR',
                     help=('Overrides for settings defined in settings.js. '
                           'May occur multiple times.'))
-  parser.add_option('-T', '--temp-dir',
+  parser.add_argument('-T', '--temp-dir',
                     default=None,
                     help=('Where to create temporary files.'))
-  parser.add_option('-v', '--verbose',
+  parser.add_argument('-v', '--verbose',
+                    default=None,
                     action='store_true',
                     dest='verbose',
                     help='Displays debug output')
-  parser.add_option('-q', '--quiet',
+  parser.add_argument('-q', '--quiet',
+                    default=None,
                     action='store_false',
                     dest='verbose',
                     help='Hides debug output')
-  parser.add_option('--suppressUsageWarning',
+  parser.add_argument('--suppressUsageWarning',
                     action='store_true',
                     default=os.environ.get('EMSCRIPTEN_SUPPRESS_USAGE_WARNING'),
                     help=('Suppress usage warning'))
+  parser.add_argument('infile', nargs='*')
 
   # Convert to the same format that argparse would have produced.
-  keywords, positional = parser.parse_args(args)
+  keywords = parser.parse_args(args)
+  positional = keywords.infile
 
   if not keywords.suppressUsageWarning:
     logging.warning('''
@@ -2222,7 +2248,7 @@ WARNING: You should normally never use this! Use emcc instead.
   if len(positional) != 1:
     raise RuntimeError('Must provide exactly one positional argument. Got ' + str(len(positional)) + ': "' + '", "'.join(positional) + '"')
   keywords.infile = os.path.abspath(positional[0])
-  if isinstance(keywords.outfile, basestring):
+  if isinstance(keywords.outfile, (type(u''), bytes)):
     keywords.outfile = open(keywords.outfile, 'w')
 
   if keywords.temp_dir is None:
