@@ -44,6 +44,13 @@ var LibraryEmbind = {
     Module['getLiveInheritedInstances'] = getLiveInheritedInstances;
     Module['flushPendingDeletes'] = flushPendingDeletes;
     Module['setDelayFunction'] = setDelayFunction;
+#if IN_TEST_HARNESS
+#if NO_DYNAMIC_EXECUTION
+    // Without dynamic execution, dynamically created functions will have no
+    // names. This lets the test suite know that.
+    Module['NO_DYNAMIC_EXECUTION'] = true;
+#endif
+#endif
   },
 
   $throwInternalError__deps: ['$InternalError'],
@@ -179,6 +186,12 @@ var LibraryEmbind = {
   $createNamedFunction__deps: ['$makeLegalFunctionName'],
   $createNamedFunction: function(name, body) {
     name = makeLegalFunctionName(name);
+#if NO_DYNAMIC_EXECUTION
+    return function() {
+      "use strict";
+      return body.apply(this, arguments);
+    };
+#else
     /*jshint evil:true*/
     return new Function(
         "body",
@@ -187,6 +200,7 @@ var LibraryEmbind = {
         "    return body.apply(this, arguments);\n" +
         "};\n"
     )(body);
+#endif
   },
 
   embind_repr: function(v) {
@@ -524,11 +538,11 @@ var LibraryEmbind = {
     }
 
     var shift = getShiftFromSize(size);
-    
+
     var fromWireType = function(value) {
         return value;
     };
-    
+
     if (minRange === 0) {
         var bitshift = 32 - 8*size;
         fromWireType = function(value) {
@@ -536,6 +550,8 @@ var LibraryEmbind = {
         };
     }
 
+    var isUnsignedType = (name.indexOf('unsigned') != -1);
+
     registerType(primitiveType, {
         name: name,
         'fromWireType': fromWireType,
@@ -548,7 +564,7 @@ var LibraryEmbind = {
             if (value < minRange || value > maxRange) {
                 throw new TypeError('Passing a number "' + _embind_repr(value) + '" from JS side to C/C++ side to an argument of type "' + name + '", which is outside the valid range [' + minRange + ', ' + maxRange + ']!');
             }
-            return value | 0;
+            return isUnsignedType ? (value >>> 0) : (value | 0);
         },
         'argPackAdvance': 8,
         'readValueFromPointer': integerReadValueFromPointer(name, shift, minRange !== 0),
@@ -773,6 +789,11 @@ var LibraryEmbind = {
     if (!(constructor instanceof Function)) {
         throw new TypeError('new_ called with constructor type ' + typeof(constructor) + " which is not a function");
     }
+#if NO_DYNAMIC_EXECUTION
+    if (constructor === Function) {
+      throw new Error('new_ cannot create a new Function with NO_DYNAMIC_EXECUTION.');
+    }
+#endif
 
     /*
      * Previously, the following line was just:
@@ -820,6 +841,63 @@ var LibraryEmbind = {
 //       return FUNCTION_TABLE[fn];
 //    }
 
+
+    // Determine if we need to use a dynamic stack to store the destructors for the function parameters.
+    // TODO: Remove this completely once all function invokers are being dynamically generated.
+    var needsDestructorStack = false;
+
+    for(var i = 1; i < argTypes.length; ++i) { // Skip return value at index 0 - it's not deleted here.
+        if (argTypes[i] !== null && argTypes[i].destructorFunction === undefined) { // The type does not define a destructor function - must use dynamic stack
+            needsDestructorStack = true;
+            break;
+        }
+    }
+
+    var returns = (argTypes[0].name !== "void");
+
+#if NO_DYNAMIC_EXECUTION
+    var argsWired = new Array(argCount - 2);
+    return function() {
+      if (arguments.length !== argCount - 2) {
+        throwBindingError('function ' + humanName + ' called with ' + arguments.length + ' arguments, expected ' + (argCount - 2) + ' args!');
+      }
+#if EMSCRIPTEN_TRACING
+      Module.emscripten_trace_enter_context('embind::' + humanName);
+#endif
+      var destructors = needsDestructorStack ? [] : null;
+      var thisWired;
+      if (isClassMethodFunc) {
+        thisWired = argTypes[1].toWireType(destructors, this);
+      }
+      for (var i = 0; i < argCount - 2; ++i) {
+        argsWired[i] = argTypes[i + 2].toWireType(destructors, arguments[i]);
+      }
+
+      var invokerFuncArgs = isClassMethodFunc ?
+          [cppTargetFunc, thisWired] : [cppTargetFunc];
+
+      var rv = cppInvokerFunc.apply(null, invokerFuncArgs.concat(argsWired));
+
+      if (needsDestructorStack) {
+        runDestructors(destructors);
+      } else {
+        for (var i = isClassMethodFunc ? 1 : 2; i < argTypes.length; i++) {
+          var param = i === 1 ? thisWired : argsWired[i - 2];
+          if (argTypes[i].destructorFunction !== null) {
+            argTypes[i].destructorFunction(param);
+          }
+        }
+      }
+
+#if EMSCRIPTEN_TRACING
+      Module.emscripten_trace_exit_context();
+#endif
+
+      if (returns) {
+        return argTypes[0].fromWireType(rv);
+      }
+    };
+#else
     var argsList = "";
     var argsListWired = "";
     for(var i = 0; i < argCount - 2; ++i) {
@@ -837,17 +915,6 @@ var LibraryEmbind = {
     invokerFnBody += "Module.emscripten_trace_enter_context('embind::" + humanName + "');\n";
 #endif
 
-    // Determine if we need to use a dynamic stack to store the destructors for the function parameters.
-    // TODO: Remove this completely once all function invokers are being dynamically generated.
-    var needsDestructorStack = false;
-
-    for(var i = 1; i < argTypes.length; ++i) { // Skip return value at index 0 - it's not deleted here.
-        if (argTypes[i] !== null && argTypes[i].destructorFunction === undefined) { // The type does not define a destructor function - must use dynamic stack
-            needsDestructorStack = true;
-            break;
-        }
-    }
-
     if (needsDestructorStack) {
         invokerFnBody +=
             "var destructors = [];\n";
@@ -876,8 +943,6 @@ var LibraryEmbind = {
         argsListWired = "thisWired" + (argsListWired.length > 0 ? ", " : "") + argsListWired;
     }
 
-    var returns = (argTypes[0].name !== "void");
-
     invokerFnBody +=
         (returns?"var rv = ":"") + "invoker(fn"+(argsListWired.length>0?", ":"")+argsListWired+");\n";
 
@@ -911,6 +976,7 @@ var LibraryEmbind = {
 
     var invokerFunction = new_(Function, args1).apply(null, args2);
     return invokerFunction;
+#endif
   },
 
   $requireFunction__deps: ['$readLatin1String', '$throwBindingError'],
@@ -918,6 +984,16 @@ var LibraryEmbind = {
     signature = readLatin1String(signature);
 
     function makeDynCaller(dynCall) {
+#if NO_DYNAMIC_EXECUTION
+      return function() {
+          var args = new Array(arguments.length + 1);
+          args[0] = rawFunction;
+          for (var i = 0; i < arguments.length; i++) {
+            args[i + 1] = arguments[i];
+          }
+          return dynCall.apply(null, args);
+      };
+#else
         var args = [];
         for (var i = 1; i < signature.length; ++i) {
             args.push('a' + i);
@@ -929,6 +1005,7 @@ var LibraryEmbind = {
         body    += '};\n';
 
         return (new Function('dynCall', 'rawFunction', body))(dynCall, rawFunction);
+#endif
     }
 
     var fp;
@@ -973,7 +1050,7 @@ var LibraryEmbind = {
   _embind_register_function: function(name, argCount, rawArgTypesAddr, signature, rawInvoker, fn) {
     var argTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     name = readLatin1String(name);
-    
+
     rawInvoker = requireFunction(signature, rawInvoker);
 
     exposePublicSymbol(name, function() {
@@ -1208,13 +1285,14 @@ var LibraryEmbind = {
 
   $genericPointerToWireType__deps: ['$throwBindingError', '$upcastPointer'],
   $genericPointerToWireType: function(destructors, handle) {
+    var ptr;
     if (handle === null) {
         if (this.isReference) {
             throwBindingError('null is not a valid ' + this.name);
         }
 
         if (this.isSmartPointer) {
-            var ptr = this.rawConstructor();
+            ptr = this.rawConstructor();
             if (destructors !== null) {
                 destructors.push(this.rawDestructor, ptr);
             }
@@ -1234,7 +1312,7 @@ var LibraryEmbind = {
         throwBindingError('Cannot convert argument of type ' + (handle.$$.smartPtrType ? handle.$$.smartPtrType.name : handle.$$.ptrType.name) + ' to parameter type ' + this.name);
     }
     var handleClass = handle.$$.ptrType.registeredClass;
-    var ptr = upcastPointer(handle.$$.ptr, handleClass, this.registeredClass);
+    ptr = upcastPointer(handle.$$.ptr, handleClass, this.registeredClass);
 
     if (this.isSmartPointer) {
         // TODO: this is not strictly true
@@ -2109,14 +2187,14 @@ var LibraryEmbind = {
         var humanName = classType.name + '.' + fieldName;
         var desc = {
             get: function() {
-                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [getterReturnType, setterArgumentType]);
+                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [rawFieldType]);
             },
             enumerable: true,
             configurable: true
         };
         if (setter) {
             desc.set = function() {
-                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [getterReturnType, setterArgumentType]);
+                throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [rawFieldType]);
             };
         } else {
             desc.set = function(v) {
