@@ -342,6 +342,8 @@ var LibraryPThread = {
             PThread.runningWorkers.splice(PThread.runningWorkers.indexOf(worker.pthread), 1); // Not a running Worker anymore.
           } else if (d.cmd === 'objectTransfer') {
             PThread.receiveObjectTransfer(e.data);
+          } else if (e.data.target === 'setimmediate') {
+            worker.postMessage(e.data); // Worker wants to postMessage() to itself to implement setImmediate() emulation.
           } else {
             Module['printErr']("worker sent an unknown command " + d.cmd);
           }
