@@ -5,8 +5,11 @@ This is an LTO-like operation, and to avoid parsing the entire tree (we might fa
 '''
 
 import os, sys, json, subprocess, time
-import shared, js_optimizer
-from tempfiles import try_delete
+
+sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
+from tools import shared, js_optimizer
+from tools.tempfiles import try_delete
 
 js_file = sys.argv[1]
 binary_file = sys.argv[2] # mem init for js, wasm binary for wasm
@@ -40,8 +43,8 @@ def find_ctors_data(js, num):
   ctors_start, ctors_end = find_ctors(js)
   assert ctors_start > 0
   ctors_text = js[ctors_start:ctors_end]
-  all_ctors = filter(lambda ctor: ctor.endswith('()') and not ctor == 'function()' and '.' not in ctor, ctors_text.split(' '))
-  all_ctors = map(lambda ctor: ctor.replace('()', ''), all_ctors)
+  all_ctors = [ctor for ctor in ctors_text.split(' ') if ctor.endswith('()') and not ctor == 'function()' and '.' not in ctor]
+  all_ctors = [ctor.replace('()', '') for ctor in all_ctors]
   assert len(all_ctors) > 0
   ctors = all_ctors[:num]
   return ctors_start, ctors_end, all_ctors, ctors
@@ -75,14 +78,14 @@ def eval_ctors_js(js, mem_init, num):
   pre_funcs_end = asm.find('function ', pre_funcs_start)
   pre_funcs_end = asm.rfind(';', pre_funcs_start, pre_funcs_end) + 1
   pre_funcs = asm[pre_funcs_start:pre_funcs_end]
-  parts = filter(lambda x: x.startswith('var '), map(lambda x: x.strip(), pre_funcs.split(';')))
+  parts = [x for x in [x.strip() for x in pre_funcs.split(';')] if x.startswith('var ')]
   global_vars = []
   new_globals = '\n'
   for part in parts:
     part = part[4:] # skip 'var '
-    bits = map(lambda x: x.strip(), part.split(','))
+    bits = [x.strip() for x in part.split(',')]
     for bit in bits:
-      name, value = map(lambda x: x.strip(), bit.split('='))
+      name, value = [x.strip() for x in bit.split('=', 1)]
       if value in ['0', '+0', '0.0'] or name in [
         'STACKTOP', 'STACK_MAX', 'DYNAMICTOP_PTR',
         'HEAP8', 'HEAP16', 'HEAP32',
@@ -247,7 +250,7 @@ console.log(JSON.stringify([numSuccessful, Array.prototype.slice.call(heap.subar
     proc = subprocess.Popen(shared.NODE_JS + [temp_file], stdout=out_file_handle, stderr=err_file_handle)
     try:
       shared.jsrun.timeout_run(proc, timeout=10, full_output=True, throw_on_failure=False)
-    except Exception, e:
+    except Exception as e:
       if 'Timed out' not in str(e): raise e
       shared.logging.debug('ctors timed out\n')
       return (0, 0, 0, 0)
@@ -263,7 +266,7 @@ console.log(JSON.stringify([numSuccessful, Array.prototype.slice.call(heap.subar
 
   # out contains the new mem init and other info
   num_successful, mem_init_raw, atexits = json.loads(out_result)
-  mem_init = ''.join(map(chr, mem_init_raw))
+  mem_init = bytes(bytearray(mem_init_raw))
   total_ctors = len(all_ctors)
   if num_successful < total_ctors:
     shared.logging.debug('not all ctors could be evalled, something was used that was not safe (and therefore was not defined, and caused an error):\n========\n' + err_result + '========')
@@ -273,7 +276,7 @@ console.log(JSON.stringify([numSuccessful, Array.prototype.slice.call(heap.subar
   else:
     elements = []
     if len(atexits) > 0:
-      elements.append('{ func: function() { %s } }' % '; '.join(map(lambda x: '_atexit(' + str(x[0]) + ',' + str(x[1]) + ')', atexits)))
+      elements.append('{ func: function() { %s } }' % '; '.join(['_atexit(' + str(x[0]) + ',' + str(x[1]) + ')' for x in atexits]))
     for ctor in all_ctors[num:]:
       elements.append('{ func: function() { %s() } }' % ctor)
     new_ctors = '__ATINIT__.push(' + ', '.join(elements) + ');'
@@ -319,7 +322,7 @@ if __name__ == '__main__':
     # js path
     mem_init_file = binary_file
     if os.path.exists(mem_init_file):
-      mem_init = json.dumps(map(ord, open(mem_init_file, 'rb').read()))
+      mem_init = json.dumps(list(bytearray(open(mem_init_file, 'rb').read())))
     else:
       mem_init = []
 
@@ -350,14 +353,14 @@ if __name__ == '__main__':
     exports_start = asm.find('return {')
     exports_end = asm.find('};', exports_start)
     exports_text = asm[asm.find('{', exports_start) + 1 : exports_end]
-    exports = map(lambda x: x.split(':')[1].strip(), exports_text.replace(' ', '').split(','))
+    exports = [x.split(':')[1].strip() for x in exports_text.replace(' ', '').split(',')]
     for r in removed:
       assert r in exports, 'global ctors were exported'
-    exports = filter(lambda e: e not in removed, exports)
+    exports = [e for e in exports if e not in removed]
     # fix up the exports
     js = open(js_file).read()
     absolute_exports_start = js.find(exports_text)
-    js = js[:absolute_exports_start] + ', '.join(map(lambda e: e + ': ' + e, exports)) + js[absolute_exports_start + len(exports_text):]
+    js = js[:absolute_exports_start] + ', '.join([e + ': ' + e for e in exports]) + js[absolute_exports_start + len(exports_text):]
     open(js_file, 'w').write(js)
     # find unreachable methods and remove them
     reachable = shared.Building.calculate_reachable_functions(js_file, exports, can_reach=False)['reachable']
