@@ -1,11 +1,15 @@
 
+from __future__ import print_function
 import os, sys, subprocess, multiprocessing, re, string, json, shutil, logging, traceback
-import shared
-from js_optimizer import *
+
+sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
+from tools import shared
+from tools.js_optimizer import *
 
 DUPLICATE_FUNCTION_ELIMINATOR = path_from_root('tools', 'eliminate-duplicate-functions.js')
 
-def process_shell(js, js_engine, shell, equivalentfn_hash_info=None):
+def process_shell(js_engine, shell, equivalentfn_hash_info=None):
   suffix = '.eliminatedupes'
 
   with temp_files.get_file(suffix + '.js') as temp_file:
@@ -36,10 +40,10 @@ def run_on_chunk(command):
     if os.environ.get('EMCC_SAVE_OPT_TEMP') and os.environ.get('EMCC_SAVE_OPT_TEMP') != '0':
       saved = 'save_' + os.path.basename(filename)
       while os.path.exists(saved): saved = 'input' + str(int(saved.replace('input', '').replace('.txt', ''))+1) + '.txt'
-      print >> sys.stderr, 'running DFE command', ' '.join(map(lambda c: c if c != filename else saved, command))
+      print('running DFE command', ' '.join([c if c != filename else saved for c in command]), file=sys.stderr)
       shutil.copyfile(filename, os.path.join(shared.get_emscripten_temp_dir(), saved))
 
-    if shared.EM_BUILD_VERBOSE_LEVEL >= 3: print >> sys.stderr, 'run_on_chunk: ' + str(command)
+    if shared.EM_BUILD_VERBOSE_LEVEL >= 3: print('run_on_chunk: ' + str(command), file=sys.stderr)
 
     proc = subprocess.Popen(command, stdout=subprocess.PIPE)
     output = proc.communicate()[0]
@@ -52,7 +56,7 @@ def run_on_chunk(command):
     f = open(filename, 'wb')
     f.write(output)
     f.close()
-    if DEBUG and not shared.WINDOWS: print >> sys.stderr, '.' # Skip debug progress indicator on Windows, since it doesn't buffer well with multiple threads printing to console.
+    if DEBUG and not shared.WINDOWS: print('.', file=sys.stderr) # Skip debug progress indicator on Windows, since it doesn't buffer well with multiple threads printing to console.
     return filename
   except KeyboardInterrupt:
     # avoid throwing keyboard interrupts from a child process
@@ -60,10 +64,10 @@ def run_on_chunk(command):
   except (TypeError, ValueError) as e:
     formatted_lines = traceback.format_exc().splitlines()
 
-    print >> sys.stderr, ">>>>>>>>>>>>>>>>>"
+    print(">>>>>>>>>>>>>>>>>", file=sys.stderr)
     for formatted_line in formatted_lines:
-        print >> sys.stderr, formatted_line
-    print >> sys.stderr, "<<<<<<<<<<<<<<<<<"
+        print(formatted_line, file=sys.stderr)
+    print("<<<<<<<<<<<<<<<<<", file=sys.stderr)
 
     raise
 
@@ -75,15 +79,15 @@ def dump_equivalent_functions(passed_in_filename, global_data):
   # If we are running more than one pass, then we want to merge
   # all the hash infos into one
   if os.path.isfile(equivalent_fn_json_file):
-    print >> sys.stderr, "Merging data from current pass for {} into {}".format(passed_in_filename, equivalent_fn_json_file)
+    print("Merging data from current pass for {} into {}".format(passed_in_filename, equivalent_fn_json_file), file=sys.stderr)
     with open(equivalent_fn_json_file) as data_file:
       equivalent_fn_info = json.load(data_file)
   else:
-    print >> sys.stderr, "Writing equivalent functions for {} to {}".format(passed_in_filename, equivalent_fn_json_file)
+    print("Writing equivalent functions for {} to {}".format(passed_in_filename, equivalent_fn_json_file), file=sys.stderr)
 
   # Merge the global data's fn_hash_to_fn_name structure into
   # the equivalent function info hash.
-  for fn_hash, fn_names in global_data['fn_hash_to_fn_name'].iteritems():
+  for fn_hash, fn_names in global_data['fn_hash_to_fn_name'].items():
     if fn_hash not in equivalent_fn_info:
       # Exclude single item arrays as they are of no use to us.
       if len(fn_names) > 1:
@@ -110,7 +114,7 @@ def write_equivalent_fn_hash_to_file(f, json_files, passed_in_filename):
 
       # Merge the data's fn_hash_to_fn_name structure into
       # the global data hash.
-      for fn_hash, fn_names in data['fn_hash_to_fn_name'].iteritems():
+      for fn_hash, fn_names in data['fn_hash_to_fn_name'].items():
         if fn_hash not in global_data['fn_hash_to_fn_name']:
             global_data['fn_hash_to_fn_name'][fn_hash] = fn_names[:]
             global_data['fn_hash_to_fn_body'][fn_hash] = data['fn_hash_to_fn_body'][fn_hash]
@@ -123,7 +127,7 @@ def write_equivalent_fn_hash_to_file(f, json_files, passed_in_filename):
 
       # Merge the data's variable_names structure into
       # the global data hash.
-      for variable, value in data['variable_names'].iteritems():
+      for variable, value in data['variable_names'].items():
         if variable not in global_data['variable_names']:
             global_data['variable_names'][variable] = value
 
@@ -131,7 +135,7 @@ def write_equivalent_fn_hash_to_file(f, json_files, passed_in_filename):
 
   # Lets generate the equivalent function hash from the global data set
   equivalent_fn_hash = {}
-  for fn_hash, fn_names in global_data['fn_hash_to_fn_name'].iteritems():
+  for fn_hash, fn_names in global_data['fn_hash_to_fn_name'].items():
     shortest_fn = None
     for fn_name in fn_names:
       if (fn_name not in variable_names) and (shortest_fn is None or (len(fn_name) < len(shortest_fn))):
@@ -184,13 +188,17 @@ def run_on_js(filename, gen_hash_info=False):
     # We need to split out the asm shell as well, for minification
     pre = js[:start_asm + len(start_asm_marker)]
     post = js[end_asm:]
-    asm_shell = js[start_asm + len(start_asm_marker):start_funcs + len(start_funcs_marker)] + '''
-EMSCRIPTEN_FUNCS();
-''' + js[end_funcs + len(end_funcs_marker):end_asm + len(end_asm_marker)]
+    asm_shell_pre = js[start_asm + len(start_asm_marker):start_funcs + len(start_funcs_marker)]
+    # Prevent "uglify" from turning 0.0 into 0 in variables' initialization. To do this we first replace 0.0 with
+    # ZERO$DOT$ZERO and then replace it back.
+    asm_shell_pre = re.sub(r'(\S+\s*=\s*)0\.0', r'\1ZERO$DOT$ZERO', asm_shell_pre)
+    asm_shell_post = js[end_funcs + len(end_funcs_marker):end_asm + len(end_asm_marker)]
+    asm_shell = asm_shell_pre + '\nEMSCRIPTEN_FUNCS();\n' + asm_shell_post
     js = js[start_funcs + len(start_funcs_marker):end_funcs]
 
     # we assume there is a maximum of one new name per line
-    asm_shell_pre, asm_shell_post = process_shell(js, js_engine, asm_shell, equivalentfn_hash_info).split('EMSCRIPTEN_FUNCS();');
+    asm_shell_pre, asm_shell_post = process_shell(js_engine, asm_shell, equivalentfn_hash_info).split('EMSCRIPTEN_FUNCS();')
+    asm_shell_pre = re.sub(r'(\S+\s*=\s*)ZERO\$DOT\$ZERO', r'\g<1>0.0', asm_shell_pre)
     asm_shell_post = asm_shell_post.replace('});', '})');
     pre += asm_shell_pre + '\n' + start_funcs_marker
     post = end_funcs_marker + asm_shell_post + post
@@ -217,8 +225,8 @@ EMSCRIPTEN_FUNCS();
   chunk_size = min(MAX_CHUNK_SIZE, max(MIN_CHUNK_SIZE, total_size / intended_num_chunks))
   chunks = shared.chunkify(funcs, chunk_size)
 
-  chunks = filter(lambda chunk: len(chunk) > 0, chunks)
-  if DEBUG and len(chunks) > 0: print >> sys.stderr, 'chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(map(len, chunks)), '-', min(map(len, chunks))
+  chunks = [chunk for chunk in chunks if len(chunk) > 0]
+  if DEBUG and len(chunks) > 0: print('chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(map(len, chunks)), '-', min(map(len, chunks)), file=sys.stderr)
   funcs = None
 
   if len(chunks) > 0:
@@ -238,20 +246,20 @@ EMSCRIPTEN_FUNCS();
 
   old_filenames = filenames[:]
   if len(filenames) > 0:
-    commands = map(lambda filename: js_engine + [DUPLICATE_FUNCTION_ELIMINATOR, filename, '--gen-hash-info' if gen_hash_info else '--use-hash-info', '--no-minimize-whitespace'], filenames)
+    commands = [js_engine + [DUPLICATE_FUNCTION_ELIMINATOR, filename, '--gen-hash-info' if gen_hash_info else '--use-hash-info', '--no-minimize-whitespace'] for filename in filenames]
 
     if DEBUG and commands is not None:
-      print >> sys.stderr, [' '.join(command if command is not None else '(null)') for command in commands]
+      print([' '.join(command if command is not None else '(null)') for command in commands], file=sys.stderr)
 
     cores = min(cores, len(filenames))
     if len(chunks) > 1 and cores >= 2:
       # We can parallelize
-      if DEBUG: print >> sys.stderr, 'splitting up js optimization into %d chunks, using %d cores  (total: %.2f MB)' % (len(chunks), cores, total_size/(1024*1024.))
+      if DEBUG: print('splitting up js optimization into %d chunks, using %d cores  (total: %.2f MB)' % (len(chunks), cores, total_size/(1024*1024.)), file=sys.stderr)
       pool = shared.Building.get_multiprocessing_pool()
       filenames = pool.map(run_on_chunk, commands, chunksize=1)
     else:
       # We can't parallize, but still break into chunks to avoid uglify/node memory issues
-      if len(chunks) > 1 and DEBUG: print >> sys.stderr, 'splitting up js optimization into %d chunks' % (len(chunks))
+      if len(chunks) > 1 and DEBUG: print('splitting up js optimization into %d chunks' % (len(chunks)), file=sys.stderr)
       filenames = [run_on_chunk(command) for command in commands]
   else:
     filenames = []
@@ -278,14 +286,8 @@ EMSCRIPTEN_FUNCS();
     funcses.append(split_funcs(open(out_file).read(), False))
   funcs = [item for sublist in funcses for item in sublist]
   funcses = None
-  def sorter(x, y):
-    diff = len(y[1]) - len(x[1])
-    if diff != 0: return diff
-    if x[0] < y[0]: return 1
-    elif x[0] > y[0]: return -1
-    return 0
   if not os.environ.get('EMCC_NO_OPT_SORT'):
-    funcs.sort(sorter)
+    funcs.sort(key=lambda x: (len(x[1]), x[0]), reverse=True)
 
   for func in funcs:
     f.write(func[1])
@@ -312,7 +314,7 @@ def save_temp_file(file_to_process):
     if not os.path.exists(os.path.dirname(destinationFile)):
       os.makedirs(os.path.dirname(destinationFile))
 
-    print >> sys.stderr, "Copying {} to {}".format(file_to_process, destinationFile)
+    print("Copying {} to {}".format(file_to_process, destinationFile), file=sys.stderr)
     shutil.copyfile(file_to_process, destinationFile)
 
 def get_func_names(javascript_file):
@@ -338,13 +340,13 @@ def eliminate_duplicate_funcs(file_name):
     # Remove previous log file if it exists
     equivalent_fn_json_file = file_name + ".equivalent_functions.json"
     if os.path.isfile(equivalent_fn_json_file):
-      print >> sys.stderr, "Deleting old json: " + equivalent_fn_json_file
+      print("Deleting old json: " + equivalent_fn_json_file, file=sys.stderr)
       os.remove(equivalent_fn_json_file)
 
     old_funcs = get_func_names(file_name)
 
   for pass_num in range(shared.Settings.ELIMINATE_DUPLICATE_FUNCTIONS_PASSES):
-    if DEBUG: print >> sys.stderr, "[PASS {}]: eliminating duplicate functions in: {}.".format(pass_num, file_name)
+    if DEBUG: print("[PASS {}]: eliminating duplicate functions in: {}.".format(pass_num, file_name), file=sys.stderr)
 
     # Generate the JSON for the equivalent hash first
     processed_file = run_on_js(filename=file_name, gen_hash_info=True)
@@ -363,7 +365,7 @@ def eliminate_duplicate_funcs(file_name):
     new_funcs = get_func_names(file_name)
 
     eliminated_funcs_file = file_name + ".eliminated_functions.json"
-    print >> sys.stderr, "Writing eliminated functions to file: {}".format(eliminated_funcs_file)
+    print("Writing eliminated functions to file: {}".format(eliminated_funcs_file), file=sys.stderr)
 
     with open(eliminated_funcs_file, 'w') as fout:
       eliminated_functions = list(set(old_funcs)-set(new_funcs))
